//
//  ExperienceViewController.m
//  HackXMLDoc
//
//  Created by Andrei Rotenstein on 12/02/10.
//  Copyright 2010 Andrei Rotenstein. All rights reserved.
//

#import "ExperienceViewController.h"
#import "WaypointDataModel.h"

#define DEBUG "YES";

@implementation ExperienceViewController

@synthesize mapView;
@synthesize locationManager;
@synthesize tryUpdatingLocation;
@synthesize theDataModel;
@synthesize moviePlayer;
@synthesize mp;
@synthesize isMediaPlaying;
@synthesize isFirstUpdate;
@synthesize button;
@synthesize movieList;
@synthesize fullscreen;
@synthesize isButtonClicked;
@synthesize buttonExists;


/*
- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    if (self = [super initWithStyle:style]) {
    }
    return self;
}
*/




- (void)viewDidLoad {
    
    // Use this to select whether TabBars are visible on the bottom
    fullscreen = YES;
    buttonExists = false;
	
	[self setIsMediaPlaying:FALSE];
	[self setIsFirstUpdate:TRUE];
	
	self.locationManager = [[CLLocationManager alloc] init]; 
	[[self locationManager] setDelegate:self];
		
	[[self mapView] setShowsUserLocation:TRUE];
	[[self mapView] setMapType:MKMapTypeHybrid];
	[[self mapView] setDelegate:self];
    [[UIApplication sharedApplication] setStatusBarHidden:FALSE];
	
	[self setTheDataModel:[WaypointDataModel sharedInstance]];
    
        
    if (isFirstUpdate) {
      movieList = [[NSMutableArray alloc]init];
        
        for (int i = 0; i <[theDataModel count]; ++ i){
            [movieList addObject:[NSNumber numberWithInt:0]];
    
        }
              //  NSLog(@"%@", movieList);
       // NSLog(@"movie list init with capacity: %d", [theDataModel count]);
    }

	// set the current Experience ViewController object to be the listener of all movieplayer events
		
	/*
	// Register to receive a notification that the movie is now in memory and ready to play
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(moviePreloadDidFinish:) 
												 name:MPMoviePlayerLoadStateDidChangeNotification 
											   object:nil];
	*/
	
	
	// Register to receive a notification when the movie scaling mode has changed. 
	[[NSNotificationCenter defaultCenter] addObserver:self 
											 selector:@selector(movieScalingModeDidChange:) 
												 name:MPMoviePlayerScalingModeDidChangeNotification 
											   object:nil];
	
	// add annotations to the mapView
	
	for( int i=0; i< [theDataModel count]; i++ )
	{
		[[self mapView]
		 addAnnotation:[[self theDataModel] locationFromIndex:i]
		];
		
	}
    
  	[super viewDidLoad];
	[self.locationManager release];
}



/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/

- (void)viewDidAppear:(BOOL)animated {
	
	[[self locationManager] startUpdatingLocation];
	NSLog(@"Updating locations now.");
	[self setTryUpdatingLocation:TRUE];
    
   
    [super viewDidAppear:animated];
	
}

 
- (void) createButton {
  
    CGRect buttonFrame = CGRectMake( self.mapView.frame.size.width-54.0,  self.mapView.frame.size.height-54.0, 44, 44);
    UIImage *btnImage = [UIImage imageNamed:@"refreshIcon.png"];
    
    button = [[UIButton alloc] initWithFrame: buttonFrame];
    [button setImage:btnImage forState: UIControlStateNormal];
    [button addTarget: self
               action: @selector(buttonClicked:)
     forControlEvents: UIControlEventTouchDown];
    [mapView addSubview: button];
    buttonExists = true;
    isButtonClicked = false;
   // NSLog(@"New button Drawn!!!!!!");
    
}

- (void) removeMyButton {
    
    [button removeFromSuperview];
    [button release];
    buttonExists = false;
    
}


- (void) buttonClicked: (id)sender
{
    if (isButtonClicked == false){
    
        NSLog( @"Button clicked." );
    
        [movieList removeAllObjects];
    
        for (int i = 0; i <[theDataModel count]; ++ i){
            [movieList addObject:[NSNumber numberWithInt:0]];
        
        }

    [self removeMyButton];
        isButtonClicked = true;
    }
   
}

- (void)viewWillDisappear:(BOOL)animated {
	NSLog(@"STOPPED Updating locations.");
	[[self locationManager] stopUpdatingLocation];
    
   
	[super viewWillDisappear:animated];
}

- (void)doUserExperienceForNewLocation:(CLLocation*) newLocation {

	// skip updating if a movie is already playing or has already played once
	if( isMediaPlaying )
		return;
	
	// Initialize a movie player object with the specified URL
	for( int i=0; i<[theDataModel count]; i++ )
	{
		CLLocation* candidateLocation = 
		[ [CLLocation alloc] 
			initWithCoordinate: [[[self theDataModel] locationFromIndex:i] coordinate]
			altitude:[newLocation altitude] 
			horizontalAccuracy:[newLocation horizontalAccuracy] 
			verticalAccuracy:[newLocation verticalAccuracy]
			timestamp:[newLocation timestamp]
		];
		float radius = [[[self theDataModel] locationFromIndex:i] radius];
		
		if( [newLocation distanceFromLocation:candidateLocation] <= radius )
		{
            // if the movie has already been played abort until refresh is pressed.
            
            if ([[movieList objectAtIndex:i] integerValue] == 1 )
            {
           //     NSLog(@"movie at %d has already been played", i);
                return;
            
            }
            [movieList replaceObjectAtIndex:i withObject:[NSNumber numberWithInt:1]];
           // [movieList replaceObjectAtIndex:[NSNumber numberWithInt:1] atIndex:i];
            
            
            //  NSLog(@"%@", movieList);
            
			NSURL* movieURL = nil;
			NSBundle* bundle = [NSBundle mainBundle];
			if (bundle) 
			{
				NSString* moviePath = 
				[bundle 
				 pathForResource:
					[[[[self theDataModel] locationFromIndex:i] mediaURL] absoluteString]
				 ofType:@"m4v"
				];
				if (moviePath)
				{
					movieURL = [NSURL fileURLWithPath:moviePath];
				}
			}
			
			if (movieURL)
			{
				mp = [[MPMoviePlayerController alloc] initWithContentURL: movieURL];
				 
				if (mp) {
					//this tells us if it's 3.2(4.0)+ or 3.1
				if ([mp respondsToSelector:@selector(loadState)])
				{
					[self setIsMediaPlaying:true];
					
					// Set movie player layout
					[mp setControlStyle:MPMovieControlStyleFullscreen];
					[mp setFullscreen:YES animated:YES];
					
					
					// May help to reduce latency
					[mp prepareToPlay];
                 
					
					// Register that the load state changed (movie is ready)
					[[NSNotificationCenter defaultCenter] addObserver:self 
															 selector:@selector(moviePlayerLoadStateChanged:) 
																 name:MPMoviePlayerLoadStateDidChangeNotification 
															   object:nil];
					
					
					// Register to receive a notification when the movie has finished playing. 
					[[NSNotificationCenter defaultCenter] addObserver:self 
															 selector:@selector(moviePlayBackDidFinish:) 
																 name:MPMoviePlayerPlaybackDidFinishNotification 
															   object:nil];
				
					// skip the rest of the loop
					return;
				}
				else
				{
					//means it's 3.1
					// save the movie player object
					[self setMoviePlayer: mp];
					
					// Apply the user specified settings to the movie player object
					[self setMoviePlayerUserSettings];
					
					// Register to receive a notification when the movie is in memory and ready to play.
					[[NSNotificationCenter defaultCenter] addObserver:self 
															 selector:@selector(moviePreloadDidFinish:) 
																 name:MPMoviePlayerContentPreloadDidFinishNotification 
															   object:nil];
					
				}
				}
			} else {
		//	NSLog(@"Movie not found: have you added your .m4v files to the project correctly?");
			}
		}
	}
}

- (void) locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)newHeading {	
	
}

- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
	// Ignore this if we shoudn't try to update the location
	if (![self tryUpdatingLocation] )
		return;
	
	//NSLog(@"New Location: %@", [newLocation description] );

	// test that the horizontal accuracy does not indicate an invalid measurement
    if (newLocation.horizontalAccuracy < 0) return;

	if( isFirstUpdate )
	{
        CLLocationCoordinate2D theCoordinate;
        theCoordinate.latitude = oldLocation.coordinate.latitude;
        theCoordinate.longitude = oldLocation.coordinate.longitude;
       // theCoordinate.latitude = 00.00;
       // theCoordinate.longitude = 00.00;
        
        
        MKCoordinateRegion viewRegion = MKCoordinateRegionMakeWithDistance(theCoordinate, 1000, 1000);
        MKCoordinateRegion adjustedRegion = [self.mapView regionThatFits:viewRegion];
        [self.mapView setRegion:adjustedRegion animated:NO];
       
        /*
		MKCoordinateRegion myCoordinateRegion;
		
		
		// set displayed region so that my location is in the centre and the map extends show all coordinates in the XML file
		myCoordinateRegion.span.latitudeDelta = 0.005;
		myCoordinateRegion.span.longitudeDelta = 0.005;
        
        myCoordinateRegion.center = [newLocation coordinate];
				
		[mapView setRegion:myCoordinateRegion animated:YES];
   */
		[self setIsFirstUpdate:false];
	}
	else
	{
		[[self mapView] setCenterCoordinate:[newLocation coordinate] animated:YES]; 
		
	}
	
	[self doUserExperienceForNewLocation:newLocation];

}

- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    // The location "unknown" error simply means the manager is currently unable to get the location.
    if ([error code] != kCLErrorLocationUnknown) {
        [self stopUpdatingLocation:NSLocalizedString(@"Error", @"Error")];
    }
}

- (void)stopUpdatingLocation:(NSString*) errorString {
	[[self locationManager] stopUpdatingLocation];
	[self setTryUpdatingLocation:false];
}


/* BEGIN MoviePlayer notification listener methods */

/*---------------------------------------------------------------------------
 * For 3.1.x devices
 * For 3.2 and 4.x see moviePlayerLoadStateChanged: 
 *--------------------------------------------------------------------------*/

//  Notification called when the movie finished preloading.
- (void) moviePreloadDidFinish:(NSNotification*)notification
{
	/* 
	 < add your code here >
	 
	 MPMoviePlayerController* moviePlayerObj=[notification object];
	 etc.
	
	 */
	//try to get rid of menubars
	mp.controlStyle =  MPMovieControlStyleNone;
	
	// Remove observer
	[[NSNotificationCenter 	defaultCenter] 
	 removeObserver:self
	 name:MPMoviePlayerContentPreloadDidFinishNotification
	 object:nil];
	
	// Play the movie
	[mp play];
	

}

/*---------------------------------------------------------------------------
 * For 3.2 and 4.x devices
 * For 3.1.x devices see moviePreloadDidFinish:
 *--------------------------------------------------------------------------*/
- (void) moviePlayerLoadStateChanged:(NSNotification*)notification 
{
	// Unless state is unknown, start playback
	if ([mp loadState] != MPMovieLoadStateUnknown)
	{
		// Remove observer
		[[NSNotificationCenter defaultCenter] 
		 removeObserver:self
		 name:MPMoviePlayerLoadStateDidChangeNotification 
		 object:nil];
        
		 [[UIApplication sharedApplication] setStatusBarHidden:TRUE];
        
            /*
        //if it's not an iPad
        if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad){
            // When tapping movie, status bar will appear, it shows up
            // in portrait mode by default. Set orientation to landscape
            [[UIApplication sharedApplication] setStatusBarHidden:TRUE];
          //  [[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:NO];
            
			 //To ensure it plays over the tabs, set it as a subview to the parentviewcontroller
              [[[self parentViewController]view] setBounds:CGRectMake(0, 0, [UIScreen mainScreen].applicationFrame.size.height, [UIScreen mainScreen].applicationFrame.size.width)];
			 [[[self parentViewController]view] setCenter:CGPointMake([UIScreen mainScreen].applicationFrame.size.width/2, [UIScreen mainScreen].applicationFrame.size.height/2)];
			 [[[self parentViewController]view] setTransform:CGAffineTransformMakeRotation(M_PI / 2)];
            
            // Set frame of movie player
            [[mp view] setFrame:CGRectMake(0, 0, [UIScreen mainScreen].applicationFrame.size.height, [UIScreen mainScreen].applicationFrame.size.width)];
            
        } else {
            // When tapping movie, status bar will appear, it shows up
            // in portrait mode by default. Set orientation to landscape
            [[UIApplication sharedApplication] setStatusBarHidden:TRUE];
            [[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationPortrait animated:NO];
            

            
            [[[self parentViewController]view] setBounds:CGRectMake(0, 0, [UIScreen mainScreen].applicationFrame.size.width, [UIScreen mainScreen].applicationFrame.size.height)];
			// [[[self parentViewController]view] setCenter:CGPointMake(160, 218)];
            [[[self parentViewController]view] setCenter:CGPointMake([UIScreen mainScreen].applicationFrame.size.width/2, [UIScreen mainScreen].applicationFrame.size.height/2)];
            // Set frame of movie player
            [[mp view] setFrame:CGRectMake(0, 0, [UIScreen mainScreen].applicationFrame.size.width, [UIScreen mainScreen].applicationFrame.size.height)];
        }
        
        */
        
	
		 
		// }
        mp.view.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
        [mp.view setFrame:[[self parentViewController]view].bounds];
        
        [[[[UIApplication sharedApplication] delegate] window] rootViewController].tabBarController.tabBar.hidden = YES;
		
		// Add movie player to parent of self (MKMapkit) as subview once it's defined
		[[[self parentViewController]view] addSubview:[mp view]];
		
		
		//get rid of menubars
		mp.controlStyle =  MPMovieControlStyleNone;
		
		// Play the movie
		[mp play];
        
        mp.controlStyle = MPMovieControlStyleFullscreen;
		/*
		if (UI_USER_INTERFACE_IDIOM() != UIUserInterfaceIdiomPad){
		mp.controlStyle = MPMovieControlStyleFullscreen;
        } else {
           mp.controlStyle = MPMovieControlStyleFullscreen;
        }
         */
	}
}


//  Notification called when the movie finished playing.
- (void) moviePlayBackDidFinish:(NSNotification*)notification
{
    /*     
	 < add your code here >
	 
	 MPMoviePlayerController* moviePlayerObj=[notification object];
	 etc.
	 */
	
	// Remove notifications
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:MPMoviePlayerPlaybackDidFinishNotification
												  object:nil];
	
	if( mp != nil ){
		[mp stop];
        
         [[[[UIApplication sharedApplication] delegate] window] rootViewController].tabBarController.tabBar.hidden = NO;
        
        [[UIApplication sharedApplication] setStatusBarHidden:FALSE];
        /*
		[[UIApplication sharedApplication] setStatusBarOrientation:UIInterfaceOrientationPortrait animated:NO];
		[[UIApplication sharedApplication] setStatusBarHidden:FALSE];
		
	
        //This works for stautus bar when visible, does it work when not?  Check if map should draw under stautus bar or behind
        rect = [[UIApplication sharedApplication] statusBarFrame];
     
        [[[self parentViewController]view] setCenter:CGPointMake([UIScreen mainScreen].applicationFrame.size.width/2, [UIScreen mainScreen].applicationFrame.size.height/2+rect.size.height)];
        [[[self parentViewController]view] setTransform:CGAffineTransformMakeRotation(degreesToRadians(0))];
     
      //to block out tabbars change height to +100
              [[[self parentViewController]view] setBounds:CGRectMake(0, 0, [UIScreen mainScreen].applicationFrame.size.width, [UIScreen mainScreen].applicationFrame.size.height)];
        */
        
        
      
        
			 [mp.view removeFromSuperview];
            [self viewDidAppear:YES];
			 
		// }
			 
			 //prevent it from activating immediately 2.5 second delay
			 [self performSelector:@selector(setAfterDelay) withObject:nil afterDelay:2.5];
        
        
        
			 [mp release];
        if (!buttonExists){
             [self createButton];
        }
	}
        
	

	
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
    
    NSLog(@"Rotation did happen.");
    
    if (buttonExists)
    {
        [button setFrame: CGRectMake( self.mapView.frame.size.width-54.0,  self.mapView.frame.size.height-54.0, 44, 44)];
    }
}


-(void)setAfterDelay{
		[self setIsMediaPlaying:FALSE];
}


//  Notification called when the movie scaling mode has changed.
- (void) movieScalingModeDidChange:(NSNotification*)notification
{
    /* 
	 < add your code here >
	 
	 MPMoviePlayerController* moviePlayerObj=[notification object];
	 etc.
	 */
}


/* END MoviePlayer notification listener methods */

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

- (void)dealloc {
    [super dealloc];
}


@end

