//
//  WaypointDataModel
//  HackXMLDoc
//
//  Created by Andrei Rotenstein on 03/02/10.
//  Copyright 2010 Andrei Rotenstein. All rights reserved.
//

#import "WaypointDataModel.h"

static WaypointDataModel* sharedInstance=nil;

@implementation WaypointDataModel

#pragma mark -
#pragma mark class instance methods

#pragma mark -
#pragma mark Singleton methods

@synthesize currentStringValue;
@synthesize currentSiteName;
@synthesize currentItem;
@synthesize locations;

// Methods implemented to support singleton behaviour.

+ (WaypointDataModel*)sharedInstance
{
    @synchronized(self)
    {
        if (sharedInstance == nil)
		{
			sharedInstance = [[WaypointDataModel alloc] init];
		}
    }
    return sharedInstance;
}

+ (id) alloc
{
	@synchronized(self)
	{
		if( sharedInstance != nil )
			return nil;
		
		if(( sharedInstance = [[super alloc] init] ))
		{
			// Create containers to hold location data.
			[sharedInstance setLocations:[[NSMutableArray arrayWithCapacity:2048] init]];
			[sharedInstance setCurrentStringValue: [[NSString alloc] init]];
			[sharedInstance setCurrentSiteName: [[NSString alloc] init]];
			[sharedInstance setCurrentItem: [[LocationItem alloc] init]];
			
			// Parse XML file.
			NSURL *url = [NSURL fileURLWithPath:[[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"mapPoints.xml"]];
			NSXMLParser* parser = [[NSXMLParser alloc] initWithContentsOfURL:url];
			[parser setDelegate:sharedInstance];
			[parser parse];
		}
	}
	return sharedInstance;
}

- (id) init
{
	return self;
}

- (id)retain {
    return self;
}

- (unsigned)retainCount {
    return UINT_MAX;  // denotes an object that cannot be released
}

- (void)release {
    //do nothing
}

- (id)autorelease {
    return self;
}


- (void)dealloc {
	// refuse to release memory.
}

// end Methods to support singleton behavior

- (int) count
{
	return [locations count];
}

- (LocationItem*) locationFromIndex:(NSUInteger)index
{
	return [locations objectAtIndex:index];
}



//
// NSXMLParser delegate methods
//

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {

	if ( [elementName isEqualToString:@"site"]) {
        return;
		
    }

	if( [elementName isEqualToString:@"location"] ) {
		currentItem = [LocationItem alloc];
		[currentItem init];
		return;
	}
	
    if ( [elementName isEqualToString:@"desc"] ) {
        return;
    }
	
    if ( [elementName isEqualToString:@"lat"] ) {
		[currentItem setLatitude:-1];
        return;
    }

    if ( [elementName isEqualToString:@"long"] ) {
		[currentItem setLongitude:-1];
        return;
    }

    if ( [elementName isEqualToString:@"rad"] ) {
        [currentItem setRadius:-1];
        return;
    }

    if ( [elementName isEqualToString:@"media"] ) {
		[currentItem setMediaURL:[[NSURL new] initWithString:@"N/A"]];
        return;
    }
}


- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
    [self setCurrentStringValue:[string copy]];
}

 
- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	
	// ignore root and empty elements
    if ( [elementName isEqualToString:@"site"]) return;
    if ( [elementName isEqualToString:@"location"]) {
		[locations addObject:[currentItem retain]];
	};
	
    if ( [elementName isEqualToString:@"desc"] ) {
		[currentItem setDescription: currentStringValue];
        return;
    }

	if ( [elementName isEqualToString:@"lat"] ) { 
		[ currentItem setLatitude: [currentStringValue floatValue] ];
        return;
    }
	
	if ( [elementName isEqualToString:@"long"] ) {
		[ currentItem setLongitude: [currentStringValue floatValue] ];
        return;
    }
	
	if ( [elementName isEqualToString:@"rad"] ) {
		[ currentItem setRadius: [currentStringValue floatValue] ];
        return;
    }
	
	if ( [elementName isEqualToString:@"media"] ) {
		[currentItem setMediaURL:[[NSURL alloc] initWithString:currentStringValue]];
        return;
    }
}


@end
